@@ -51,6 +51,9 @@
 #include <libgnome/gnome-macros.h>
 
 #include "gnome-calc.h"
+#include "sr.h"
+
+#undef GNOME_CALC_DEBUG
 
 typedef void (*sighandler_t)(int);
 
@@ -129,6 +132,8 @@ enum {
 
 static gint gnome_calc_signals[LAST_SIGNAL];
 
+static gboolean cmd_saved = FALSE;
+
 GNOME_CLASS_BOILERPLATE (GnomeCalc, gnome_calc,
 			 GtkVBox, GTK_TYPE_VBOX)
 
@@ -157,7 +162,7 @@ gnome_calc_class_init (GnomeCalcClass *class)
 	class->result_changed = NULL;
 }
 
-#if 0 /*only used for debugging*/
+#ifdef GNOME_CALC_DEBUG 
 static void
 dump_stack(GnomeCalc *gc)
 {
@@ -208,12 +213,29 @@ stack_pop(GList **stack)
 	g_free(s);
 }
 
+static void
+set_display (GnomeCalc *gc)
+{
+	GtkWidget *label = gc->_priv->text_display;
+	gchar *string;
+
+	string = g_strconcat ("<span foreground=\"white\"  font_desc=\"Helvetica 24\">", gc->_priv->result_string, "</span>", NULL);
+	gtk_label_set_markup (GTK_LABEL (label), string);
+	g_free (string);
+}
+
 static void
 do_error(GnomeCalc *gc)
 {
 	gc->_priv->error = TRUE;
-	strcpy(gc->_priv->result_string,"e");
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
+	if (errno == ERANGE)
+		strcpy(gc->_priv->result_string,_("inf"));
+	else
+		strcpy(gc->_priv->result_string,"e");
+
+	set_display (gc);
+
+
 }
 
 /*we handle sigfpe's so that we can find all the errors*/
@@ -281,8 +303,9 @@ reduce_stack(GnomeCalc *gc)
 
 	if(errno>0 ||
 	   finite(stack->d.number)==0) {
-		errno = 0;
-		do_error(gc);
+	   	do_error(gc);
+		stack->d.number = 0;
+		errno = 0;		
 	}
 }
 
@@ -398,8 +421,7 @@ set_result(GnomeCalc *gc)
 	strncpy(gc->_priv->result_string,buf,12);
 	gc->_priv->result_string[12]='\0';
 
-	gtk_entry_set_text (GTK_ENTRY(gc->_priv->text_display), gc->_priv->result_string);
-
+	set_display (gc);
 	gtk_signal_emit(GTK_OBJECT(gc),
 			gnome_calc_signals[RESULT_CHANGED_SIGNAL],
 			gc->_priv->result);
@@ -477,7 +499,10 @@ convert_num(gdouble num, GnomeCalcMode from, GnomeCalcMode to)
 static void
 no_func(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GList *list;
+	static CalculatorStack saved_func, saved_num;
+	CalculatorStack *stack1, *stack2, *add_func, *add_num;
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 
 	g_return_if_fail(gc!=NULL);
 
@@ -493,7 +518,55 @@ no_func(GtkWidget *w, gpointer data)
 	}
 
 	reduce_stack_prec(gc,NULL);
-	if(gc->_priv->error) return;
+
+	if(gc->_priv->error) {
+		gc->_priv->error = FALSE;
+		return;
+	}
+
+	/* = after <math_func>=  eg: 3+== */
+	if (cmd_saved == TRUE) {
+		add_func = g_new(CalculatorStack, 1);
+		add_func->type = saved_func.type;
+		add_func->d = saved_func.d;
+		gc->_priv->stack = g_list_prepend(gc->_priv->stack, add_func);
+
+		add_num = g_new(CalculatorStack, 1);
+		add_num->type = saved_num.type;
+		add_num->d = saved_num.d;
+		gc->_priv->stack = g_list_prepend(gc->_priv->stack, add_num);
+		gc->_priv->add_digit = FALSE;
+
+		reduce_stack(gc);
+	} else {
+		if(gc->_priv->stack == NULL)
+			return;
+		stack1 = gc->_priv->stack->data;
+
+		list=g_list_next(gc->_priv->stack);
+		if(list) {
+			stack2 = list->data;
+
+			if( stack1->type == CALCULATOR_FUNCTION && 
+		    		stack2->type == CALCULATOR_NUMBER) {
+				saved_func.type = CALCULATOR_FUNCTION;
+				saved_func.d = stack1->d;
+
+				saved_num.type = CALCULATOR_NUMBER;
+				saved_num.d = stack2->d;
+
+				add_num = g_new(CalculatorStack,1);
+				add_num->type = saved_num.type;
+				add_num->d = saved_num.d;
+				gc->_priv->stack = g_list_prepend(gc->_priv->stack, add_num);
+				gc->_priv->add_digit = FALSE;
+
+				reduce_stack(gc);
+				cmd_saved = TRUE;
+			}
+		}
+	}
+
 	set_result(gc);
 
 	unselect_invert(gc);
@@ -502,7 +575,7 @@ no_func(GtkWidget *w, gpointer data)
 static void
 simple_func(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 	CalculatorStack *stack;
 	CalculatorButton *but = data;
 	MathFunction1 func = but->data;
@@ -511,6 +584,8 @@ simple_func(GtkWidget *w, gpointer data)
 	g_return_if_fail(func!=NULL);
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
@@ -522,6 +597,12 @@ simple_func(GtkWidget *w, gpointer data)
 	}
 
 	stack = gc->_priv->stack->data;
+
+	if(stack->type==CALCULATOR_FUNCTION) {
+		stack_pop(&gc->_priv->stack);
+		stack = gc->_priv->stack->data;
+	}
+
 	if(stack->type!=CALCULATOR_NUMBER) {
 		unselect_invert(gc);
 		return;
@@ -545,8 +626,10 @@ simple_func(GtkWidget *w, gpointer data)
 	
 	if(errno>0 ||
 	   finite(stack->d.number)==0) {
+	   	do_error(gc);
+		gc->_priv->error = FALSE;
+		stack->d.number = 0;
 		errno = 0;
-		do_error(gc);
 		return;
 	}
 	
@@ -564,7 +647,7 @@ simple_func(GtkWidget *w, gpointer data)
 static void
 math_func(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 	CalculatorStack *stack;
 	CalculatorButton *but = data;
 	MathFunction2 func = but->data;
@@ -573,6 +656,8 @@ math_func(GtkWidget *w, gpointer data)
 	g_return_if_fail(func!=NULL);
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
@@ -588,6 +673,12 @@ math_func(GtkWidget *w, gpointer data)
 	set_result(gc);
 
 	stack = gc->_priv->stack->data;
+
+	if(stack->type==CALCULATOR_FUNCTION) {
+		stack_pop(&gc->_priv->stack);
+		stack = gc->_priv->stack->data;
+	}
+
 	if(stack->type!=CALCULATOR_NUMBER) {
 		unselect_invert(gc);
 		return;
@@ -610,12 +701,14 @@ reset_calc(GtkWidget *w, gpointer data)
 {
 	GnomeCalc *gc;
 	if(w)
-		gc = gtk_object_get_user_data(GTK_OBJECT(w));
+		gc = g_object_get_data(G_OBJECT(w), "set_data");
 	else
 		gc = data;
 
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	while(gc->_priv->stack)
 		stack_pop(&gc->_priv->stack);
 
@@ -637,10 +730,12 @@ reset_calc(GtkWidget *w, gpointer data)
 static void
 clear_calc(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	/* if in add digit mode, just clear the number, otherwise clear
 	 * state as well */
 	if(!gc->_priv->add_digit) {
@@ -683,10 +778,12 @@ gnome_calc_clear(GnomeCalc *gc, const gboolean reset)
 static void
 add_digit (GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data (GTK_OBJECT (w));
+	GnomeCalc *gc = g_object_get_data (G_OBJECT (w), "set_data");
 	CalculatorButton *but = data;
 	gchar *digit = but->name;
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
@@ -715,7 +812,7 @@ add_digit (GtkWidget *w, gpointer data)
 		else if(strlen(gc->_priv->result_string)>9)
 			return;
 		else if(gc->_priv->result_string[0]=='\0')
-			strcpy(gc->_priv->result_string," 1");
+			strcpy(gc->_priv->result_string," 0");
 	} else if(digit[0]=='.') {
 		if(strchr(gc->_priv->result_string,'.'))
 			return;
@@ -733,8 +830,8 @@ add_digit (GtkWidget *w, gpointer data)
 
 	strcat (gc->_priv->result_string, digit);
 
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
-
+	set_display (gc);
+ 	
         /* make sure get values in a consistent manner */
 	gnome_i18n_push_c_numeric_locale ();
 	sscanf(gc->_priv->result_string, "%lf", &gc->_priv->result);
@@ -751,11 +848,13 @@ c_neg(gdouble arg)
 static void
 negate_val(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 	char *p;
 
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
@@ -784,8 +883,8 @@ negate_val(GtkWidget *w, gpointer data)
 	gnome_i18n_pop_c_numeric_locale ();
 	sscanf(gc->_priv->result_string, "%lf", &gc->_priv->result);
 	gnome_i18n_push_c_numeric_locale ();
-	
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
+
+	set_display (gc);
 }
 
 static gdouble
@@ -876,8 +975,8 @@ gnome_calc_set(GnomeCalc *gc, gdouble result)
 static void
 store_m(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
-
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
+	
 	g_return_if_fail(gc!=NULL);
 
 	if(gc->_priv->error)
@@ -887,15 +986,15 @@ store_m(GtkWidget *w, gpointer data)
 
 	gc->_priv->memory = gc->_priv->result;
 
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
-
+	set_display (gc);
+ 	
 	unselect_invert(gc);
 }
 
 static void
 recall_m(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 
 	g_return_if_fail(gc!=NULL);
 
@@ -910,8 +1009,8 @@ recall_m(GtkWidget *w, gpointer data)
 static void
 sum_m(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
-
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
+	
 	g_return_if_fail(gc!=NULL);
 
 	if(gc->_priv->error)
@@ -921,15 +1020,15 @@ sum_m(GtkWidget *w, gpointer data)
 
 	gc->_priv->memory += gc->_priv->result;
 
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
-
+	set_display (gc);
+ 	
 	unselect_invert(gc);
 }
 
 static void
 exchange_m(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 
 	g_return_if_fail(gc!=NULL);
 
@@ -944,7 +1043,7 @@ exchange_m(GtkWidget *w, gpointer data)
 static void
 invert_toggle(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 
 	g_return_if_fail(gc!=NULL);
 
@@ -960,7 +1059,7 @@ invert_toggle(GtkWidget *w, gpointer data)
 static void
 drg_toggle(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 	GnomeCalcMode oldmode;
 
 	g_return_if_fail(gc!=NULL);
@@ -995,7 +1094,7 @@ drg_toggle(GtkWidget *w, gpointer data)
 static void
 set_pi(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 
 	g_return_if_fail(gc!=NULL);
 
@@ -1008,11 +1107,14 @@ set_pi(GtkWidget *w, gpointer data)
 }
 
 static void
-set_e(GtkWidget *w, gpointer data)
+set_e(GtkWidget *w, GdkEvent *event, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 
 	g_return_if_fail(gc!=NULL);
+	
+	if (event->type == GDK_3BUTTON_PRESS)
+		run_slide_rule ();
 
 	if(gc->_priv->error)
 		return;
@@ -1025,10 +1127,12 @@ set_e(GtkWidget *w, gpointer data)
 static void
 add_parenth(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
@@ -1049,9 +1153,11 @@ add_parenth(GtkWidget *w, gpointer data)
 static void
 sub_parenth(GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
@@ -1082,7 +1188,7 @@ sin_helper (double value)
 {
 	double ret = sin (value);
 	
-	if (fabs (ret) <= sin (M_PI)) 
+	if (fabs (ret) <= fabs (sin (M_PI))) 
 		return 0.0;
 	else
 		return ret;
@@ -1092,12 +1198,29 @@ static double
 cos_helper (double value)
 {
 	double ret = cos (value);
-	if (fabs (ret) <= cos (M_PI_2))
+	if (fabs (ret) <= fabs (cos (M_PI_2)))
 		return 0.0;
 	else
 		return ret;
 }
 
+static double
+tan_helper (double value)
+{
+	double ret = tan (value);
+	double temp1, temp2;
+
+	if (fabs (ret) >= fabs (tan(3 * M_PI_2))) {
+		errno=ERANGE;
+		return 0.0;
+	}
+	else if (fabs (ret) <= fabs (tan(M_PI))) {
+		return 0.0;
+	}
+	else
+		return ret;
+}
+
 static const CalculatorButton buttons[8][5] = {
 	{
 		{N_("1/x"),  (GtkSignalFunc)simple_func, c_inv,  NULL,   FALSE, {0}, N_("Inverse"), NULL },
@@ -1106,11 +1229,12 @@ static const CalculatorButton buttons[8][5] = {
 		{N_("CE/C"), (GtkSignalFunc)clear_calc,  NULL,   NULL,   FALSE, {GDK_Clear,GDK_Delete,0}, N_("Clear"), NULL },
 		{N_("AC"),   (GtkSignalFunc)reset_calc,  NULL,   NULL,   FALSE, {'a','A', GDK_Escape, 0}, N_("Reset"), NULL }
 	},{
-		{N_("INV"),  NULL,                       NULL,   NULL,   FALSE, {'i','I',0}, N_("Shift"), NULL }, /*inverse button*/
+		{N_("INV"),  NULL,                       NULL,   NULL,   FALSE, {'i','I',0}, N_("Inverse trigonometry/log function"), NULL }, /*inverse button*/
 		{N_("sin"),  (GtkSignalFunc)simple_func, sin_helper, asin, TRUE,{'s','S',0}, N_("Sine"), NULL },
 		{N_("cos"),  (GtkSignalFunc)simple_func, cos_helper, acos, TRUE,{'c','C',0}, N_("Cosine"), NULL },
-		{N_("tan"),  (GtkSignalFunc)simple_func, tan,    atan,   TRUE,  {'t','T',0}, N_("Tangent"), NULL },
-		{N_("DEG"),  (GtkSignalFunc)drg_toggle,  NULL,   NULL,   FALSE, {'d','D',0}, N_("Switch degrees / radians / grad"), NULL }
+		{N_("tan"),  (GtkSignalFunc)simple_func, tan_helper,    atan,   TRUE,  {'t','T',0}, N_("Tangent"), NULL },
+		/* Ugly hack so that window won't resize when selecting GRAD - there should be a better fix that this */
+		{N_(" DEG  "),  (GtkSignalFunc)drg_toggle,  NULL,   NULL,   FALSE, {'d','D',0}, N_("Switch degrees / radians / grad"), NULL }
 	},{
 		{N_("e"),    (GtkSignalFunc)set_e,       NULL,   NULL,   FALSE, {'e','E',0}, N_("Base of Natural Logarithm"), NULL },
 		{N_("EE"),   (GtkSignalFunc)add_digit,   "e+",   NULL,   FALSE, {0}, N_("Add digit"), NULL },
@@ -1124,25 +1248,25 @@ static const CalculatorButton buttons[8][5] = {
 		{N_(")"),    (GtkSignalFunc)sub_parenth, NULL,   NULL,   FALSE, {')',0}, N_("Closing Parenthesis"), NULL },
 		{N_("/"),    (GtkSignalFunc)math_func,   c_div,  NULL,   FALSE, {'/',GDK_KP_Divide,0}, N_("Divide by"), NULL }
 	},{
-		{N_("STO"),  (GtkSignalFunc)store_m,     NULL,   NULL,   FALSE, {0}, N_("Store"), NULL },
+		{N_("STO"),  (GtkSignalFunc)store_m,     NULL,   NULL,   FALSE, {0}, N_("Store the value in the\ndisplay field in memory"), NULL },
 		{N_("7"),    (GtkSignalFunc)add_digit,   NULL,   NULL,   FALSE, {'7',GDK_KP_7,GDK_KP_Home,0}, NULL, NULL },
 		{N_("8"),    (GtkSignalFunc)add_digit,   NULL,   NULL,   FALSE, {'8',GDK_KP_8,GDK_KP_Up,0}, NULL, NULL },
 		{N_("9"),    (GtkSignalFunc)add_digit,   NULL,   NULL,   FALSE, {'9',GDK_KP_9,GDK_KP_Page_Up,0}, NULL, NULL },
 		{N_("*"),    (GtkSignalFunc)math_func,   c_mul,  NULL,   FALSE, {'*',GDK_KP_Multiply,0}, N_("Multiply by"), NULL }
 	},{
-		{N_("RCL"),  (GtkSignalFunc)recall_m,    NULL,   NULL,   FALSE, {0}, N_("Recall"), NULL },
+		{N_("RCL"),  (GtkSignalFunc)recall_m,    NULL,   NULL,   FALSE, {0}, N_("Display the value in memory\nin the display field"), NULL },
 		{N_("4"),    (GtkSignalFunc)add_digit,   NULL,   NULL,   FALSE, {'4',GDK_KP_4,GDK_KP_Left,0}, NULL, NULL },
 		{N_("5"),    (GtkSignalFunc)add_digit,   NULL,   NULL,   FALSE, {'5',GDK_KP_5,GDK_KP_Begin,0}, NULL, NULL },
 		{N_("6"),    (GtkSignalFunc)add_digit,   NULL,   NULL,   FALSE, {'6',GDK_KP_6,GDK_KP_Right,0}, NULL, NULL },
 		{N_("-"),    (GtkSignalFunc)math_func,   c_sub,  NULL,   FALSE, {'-',GDK_KP_Subtract,0}, N_("Subtract"), NULL }
 	},{
-		{N_("SUM"),  (GtkSignalFunc)sum_m,       NULL,   NULL,   FALSE, {0}, N_("Sum"), NULL },
+		{N_("SUM"),  (GtkSignalFunc)sum_m,       NULL,   NULL,   FALSE, {0}, N_("Add the value in the display\nfield to the value in memory"), NULL },
 		{N_("1"),    (GtkSignalFunc)add_digit,   NULL,   NULL,   FALSE, {'1',GDK_KP_1,GDK_KP_End,0}, NULL, NULL },
 		{N_("2"),    (GtkSignalFunc)add_digit,   NULL,   NULL,   FALSE, {'2',GDK_KP_2,GDK_KP_Down,0}, NULL, NULL },
 		{N_("3"),    (GtkSignalFunc)add_digit,   NULL,   NULL,   FALSE, {'3',GDK_KP_3,GDK_KP_Page_Down,0}, NULL, NULL },
 		{N_("+"),    (GtkSignalFunc)math_func,   c_add,  NULL,   FALSE, {'+',GDK_KP_Add,0}, N_("Add"), NULL }
 	},{
-		{N_("EXC"),  (GtkSignalFunc)exchange_m,  NULL,   NULL,   FALSE, {0}, N_("Exchange"), NULL },
+		{N_("EXC"),  (GtkSignalFunc)exchange_m,  NULL,   NULL,   FALSE, {0}, N_("Exchange the values in the\ndisplay field and memory"), NULL },
 		{N_("0"),    (GtkSignalFunc)add_digit,   NULL,   NULL,   FALSE, {'0',GDK_KP_0,GDK_KP_Insert,0}, NULL, NULL },
 		{N_("."),    (GtkSignalFunc)add_digit,   NULL,   NULL,   FALSE, {'.',GDK_KP_Decimal,',',GDK_KP_Delete,0}, N_("Decimal Point"), NULL },
 		{N_("+/-"),  (GtkSignalFunc)negate_val,  c_neg,  NULL,   FALSE, {0}, N_("Change sign"), NULL },
@@ -1165,13 +1289,20 @@ create_button(GnomeCalc *gc, GtkWidget *table, int x, int y)
 		gc->_priv->invert_button = w;
 		gtk_signal_connect (GTK_OBJECT (w), "toggled",
 				    GTK_SIGNAL_FUNC(invert_toggle), gc);
-	} else {
+	} 
+	else if (strcmp (but->name, "e") == 0) {
+		w = gtk_button_new_with_label (_(but->name));
+		gtk_signal_connect (GTK_OBJECT (w), "button_press_event",
+				    but->signal_func,
+				    (gpointer) but);
+	}
+	else {
 		w = gtk_button_new_with_label(_(but->name));
 		gtk_signal_connect(GTK_OBJECT(w), "clicked",
 				   but->signal_func,
 				   (gpointer) but);
 	}
-
+	
 	for(i=0;but->keys[i]!=0;i++) {
 		gtk_widget_add_accelerator(w, "clicked",
 					   gc->_priv->accel,
@@ -1193,7 +1324,7 @@ create_button(GnomeCalc *gc, GtkWidget *table, int x, int y)
 		gtk_tooltips_set_tip(GTK_TOOLTIPS(gc->_priv->tooltips), w, _(but->tooltip), but->tooltip_private);
 	}
 
-	gtk_object_set_user_data(GTK_OBJECT(w),gc);
+	g_object_set_data(G_OBJECT(w), "set_data", gc);
 	gtk_widget_show(w);
 	gtk_table_attach(GTK_TABLE(table), w,
 			 x, x+1, y, y+1,
@@ -1209,26 +1340,30 @@ create_button(GnomeCalc *gc, GtkWidget *table, int x, int y)
 static void
 gnome_calc_instance_init (GnomeCalc *gc)
 {
+	GtkWidget *hbox, *event;
 	gint x,y;
 	GtkWidget *table;
-	PangoFontDescription *pdesc;
-
+	GdkColor color;
+	
 	gc->_priv = g_new0(GnomeCalcPrivate, 1);
-
-	gc->_priv->text_display = gtk_entry_new ();
-	gtk_editable_set_editable (GTK_EDITABLE (gc->_priv->text_display), FALSE);
-
-	gtk_box_pack_start (GTK_BOX (gc), gc->_priv->text_display, FALSE, FALSE, 0);
-	gtk_widget_set_direction (gc->_priv->text_display, GTK_TEXT_DIR_RTL);
-
-	pdesc = pango_font_description_from_string ("Helvetica 24");
-	if (pdesc != NULL) {
-		gtk_widget_modify_font (gc->_priv->text_display, pdesc);
-		pango_font_description_free (pdesc);
-	}
-
+	
+	event = gtk_event_box_new ();
+	gtk_widget_show (event);
+	gdk_color_parse ("#000000", &color);
+	gtk_widget_modify_bg (event, GTK_STATE_NORMAL, &color); 
+	gtk_box_pack_start (GTK_BOX (gc), event, FALSE, FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (event), 1);
+	
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_widget_show (hbox);
+	gtk_container_add (GTK_CONTAINER (event), hbox);
+	
+	gc->_priv->text_display = gtk_label_new (NULL);
+	gtk_label_set_selectable (GTK_LABEL (gc->_priv->text_display), TRUE);
+	/*gtk_label_set_justify (GTK_LABEL (gc->_priv->text_display), GTK_JUSTIFY_RIGHT);*/
 	gtk_widget_show (gc->_priv->text_display);
-
+	gtk_box_pack_end (GTK_BOX (hbox), gc->_priv->text_display, FALSE, TRUE, 0);	
+	
 	gc->_priv->stack = NULL;
 	gc->_priv->result = 0;
 	strcpy(gc->_priv->result_string," 0");
@@ -1242,15 +1377,18 @@ gnome_calc_instance_init (GnomeCalc *gc)
 	table = gtk_table_new(8,5,TRUE);
 	gtk_widget_show(table);
 
-	gtk_box_pack_end(GTK_BOX(gc),table,TRUE,TRUE,0);
+	gtk_box_pack_start(GTK_BOX(gc),table,TRUE,TRUE,0);
 
 	for(x=0;x<5;x++) {
 		for(y=0;y<8;y++) {
 			create_button(gc, table, x, y);
 		}
-	}
+	}	
+	
 	gtk_tooltips_enable (gc->_priv->tooltips);
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
+
+        set_display (gc);
+	
 }
 
 
@@ -1268,7 +1406,7 @@ gnome_calc_new (void)
 {
 	GnomeCalc *gcalculator;
 
-	gcalculator = gtk_type_new (gnome_calc_get_type ());
+	gcalculator = g_object_new (gnome_calc_get_type (), NULL);
 
 	return GTK_WIDGET (gcalculator);
 }
@@ -1378,7 +1516,7 @@ gnome_calc_get_result_string(GnomeCalc *gc)
 static void
 backspace_cb (GtkWidget *w, gpointer data)
 {
-	GnomeCalc *gc = gtk_object_get_user_data(GTK_OBJECT(w));
+	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 	char *old_locale;
 	gint length;
 
@@ -1417,9 +1555,9 @@ backspace_cb (GtkWidget *w, gpointer data)
 		if (stack->type == CALCULATOR_NUMBER)
 			stack->d.number = gc->_priv->result;
 	}
-	
-	gtk_entry_set_text (GTK_ENTRY (gc->_priv->text_display), gc->_priv->result_string);
 
+	set_display (gc);
+ 	
 	/* make sure get values in a consistent manner */
 	old_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
@@ -1492,7 +1630,7 @@ gnome_calc_bind_extra_keys (GnomeCalc *gc,
 	g_return_if_fail (GNOME_IS_CALC (gc));
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 
-	gtk_object_set_user_data (GTK_OBJECT (widget), gc);
+	g_object_set_data (G_OBJECT (widget), "set_data", gc);
 	
 	gtk_signal_connect (GTK_OBJECT (widget), "event",
 			    GTK_SIGNAL_FUNC (event_cb), gc);
