@@ -1,4 +1,4 @@
-/* $Id: dict.c,v 1.15 2002/03/10 15:33:09 kevinv Exp $ */
+/* $Id: dict.c,v 1.19 2002/05/11 21:04:27 gman Exp $ */
 /* -*- mode: c; style: k&r; c-basic-offset: 4 -*- */
 
 /*
@@ -124,6 +124,10 @@ static int
 cycle_buffer (dict_context_t *context) 
 {
     int amt_read, res;
+    GIOChannelError chanerr;
+
+    if (!context->channel)
+	return -1;
     
     if (!context->read_cycle_done) {
         if (context->write_ptr > context->read_ptr)
@@ -139,7 +143,12 @@ cycle_buffer (dict_context_t *context)
         
         if (res == G_IO_ERROR_AGAIN)
 	    context->read_cycle_done = TRUE;
-        else if (res != G_IO_ERROR_NONE)
+	else if (res == G_IO_ERROR_UNKNOWN) {
+		chanerr = g_io_channel_error_from_errno(errno);
+		if (chanerr == G_IO_CHANNEL_ERROR_PIPE || G_IO_CHANNEL_ERROR_FAILED)
+			context->channel = NULL;
+		return -1;
+        } else if (res != G_IO_ERROR_NONE)
 	    return -1;
         
         context->write_ptr += amt_read;
@@ -319,7 +328,9 @@ static void data_notify_define (dict_context_t *context, char *line)
             buf = line;
             while (!isspace (*buf)) buf++;
             *buf = '\0';
-            
+	    ++buf;
+           
+	    command->db_full_name = g_strdup (buf); 
             command->db_name = g_strdup (line);
             command->def = g_string_new (NULL);
             command->state = S_DATA;
@@ -630,7 +641,7 @@ static gboolean dict_data_notify (GIOChannel *channel, GIOCondition cond,
 	    g_free (line);
         }
     }
-    else if (cond == G_IO_ERR) {
+    else if (cond & G_IO_ERR) {
         if (context->command->error_notify_cb)
 	    context->command->error_notify_cb (context->command, 
 					       DICT_SOCKET_ERROR,
@@ -763,9 +774,8 @@ int dict_connect (dict_context_t *context)
 
 /* dict_disconnect (context)
  *
- * Opens a connection to the DICT server.
+ * Closes a connection to the DICT server.
  *
- * Returns 0 on success, -1 on connection error
  */
 
 void dict_disconnect (dict_context_t *context) 
