@@ -36,27 +36,10 @@ extern GList *actions_db;
 static GList *local_actions_db = NULL;
 
 GtkWidget *actions_dialog = NULL;
-GtkCTree *ctree = NULL;
-GtkCTreeNode *selected_node = NULL;
-GList *ctree_parent = NULL;
-
-static void
-tree_select_row (GtkCTree     *ctree,
-		 GtkCTreeNode *row,
-		 gint          column)
-{
-	selected_node = row;
-}
-
-static void
-tree_unselect_row (GtkCTree     *ctree,
-		   GtkCTreeNode *row,
-		   gint          column)
-{
-	if (selected_node == row)
-		selected_node = NULL;
-}
-
+GtkWidget *ctree_view;
+GtkTreeStore *ctree = NULL;
+GtkTreeIter *selected_node = NULL;
+GtkTreeIter *ctree_parent = NULL;
 
 static void
 apply_actions (GtkWidget *w, gpointer data)
@@ -86,39 +69,41 @@ mon_edit_actions (GtkWidget *widget, gpointer data)
 {
   GtkWidget *hbox;
   GtkWidget *button;       
-  GtkWidget *padding;       
   GtkWidget *swin;
   GtkBox *vbox;
   const gchar *title[] = {N_("Action database")};
-  
+  GtkCellRenderer *cell_renderer;
+  GtkTreeViewColumn *column; 
+
   /* Create main window ------------------------------------------------  */
   actions_dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_container_set_border_width (GTK_CONTAINER (actions_dialog), 5);
-  gtk_window_set_title (GTK_WINDOW (actions_dialog), _("Monitor options"));
+  gtk_window_set_title (GTK_WINDOW (actions_dialog), _("Action options"));
   gtk_widget_set_style (actions_dialog, cfg->main_style);
   gtk_widget_set_usize (actions_dialog, 400, -1);
   gtk_signal_connect (GTK_OBJECT (actions_dialog), "destroy",
 		      GTK_SIGNAL_FUNC (gtk_widget_destroyed),
 		      &actions_dialog);
   
-  vbox = (GtkBox *)gtk_vbox_new (FALSE, 2);
+  vbox = (GtkBox *)gtk_vbox_new (FALSE, 5);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
   gtk_container_add (GTK_CONTAINER (actions_dialog), GTK_WIDGET (vbox));
   gtk_widget_show (GTK_WIDGET (vbox)); 
   
   /* List with actions */
 
-  ctree = GTK_CTREE (gtk_ctree_new_with_titles (1, 0, (char **)title));
-  gtk_signal_connect (GTK_OBJECT (ctree), "tree_select_row",
-		      GTK_SIGNAL_FUNC (tree_select_row), NULL);
-  gtk_signal_connect (GTK_OBJECT (ctree), "tree_unselect_row",
-		      GTK_SIGNAL_FUNC (tree_unselect_row), NULL);
+  ctree = gtk_tree_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);
+  cell_renderer = gtk_cell_renderer_text_new ();
+  ctree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (ctree));
+  g_object_unref (G_OBJECT (ctree));
+  column = gtk_tree_view_column_new_with_attributes (*title, cell_renderer,
+                                                     "text", 0, NULL);
+  gtk_tree_view_append_column (GTK_TREE_VIEW (ctree_view), column);
   selected_node = NULL;
   swin = gtk_scrolled_window_new (NULL, NULL);
-  gtk_container_add (GTK_CONTAINER (swin), GTK_WIDGET (ctree));
-  gtk_ctree_set_line_style (ctree, GTK_CTREE_LINES_DOTTED);
-  gtk_clist_set_reorderable (GTK_CLIST(ctree), TRUE);
-  gtk_widget_set_usize (GTK_WIDGET (ctree), -1, 300);
+  gtk_container_add (GTK_CONTAINER (swin), GTK_WIDGET (ctree_view));
+  gtk_tree_view_set_reorderable (GTK_TREE_VIEW (ctree_view), TRUE);
+  gtk_widget_set_usize (GTK_WIDGET (ctree_view), -1, 300);
 
 /* ----------------------------------------------------------------------
   gtk_signal_connect (GTK_OBJECT (ctree), "button_press_event",
@@ -134,56 +119,53 @@ mon_edit_actions (GtkWidget *widget, gpointer data)
   -------------------------------------------------------------------- */
 
   gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (swin), TRUE, TRUE, 0);
-  gtk_clist_column_titles_passive (GTK_CLIST (ctree));
-  gtk_clist_set_column_justification (GTK_CLIST (ctree), 1, GTK_JUSTIFY_RIGHT);
-  gtk_clist_set_selection_mode (GTK_CLIST (ctree), GTK_SELECTION_SINGLE);
+  gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), FALSE);
+  gtk_tree_view_column_set_alignment ( GTK_TREE_VIEW_COLUMN (column), 0.1);
+  gtk_tree_selection_set_mode ( (GtkTreeSelection *)gtk_tree_view_get_selection
+                               (GTK_TREE_VIEW (ctree_view)),
+                                GTK_SELECTION_SINGLE);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swin),
-				  GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC);
-  gtk_clist_set_column_width (GTK_CLIST (ctree), 0, 300);
+				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_tree_view_column_set_fixed_width ( GTK_TREE_VIEW_COLUMN (column), 300);
   gtk_widget_show_all (GTK_WIDGET(swin));
       
       
   /* Make bottom part ------------------------------------------------ */
   hbox = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_start (vbox, hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
   button = gtk_button_new_with_label (_("Add..."));
   gtk_signal_connect (GTK_OBJECT (button), "clicked", 
 		      GTK_SIGNAL_FUNC (add_actions_entry_cb), NULL);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
   
   button = gtk_button_new_with_label (_("Edit..."));
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (button), "clicked", 
 		      GTK_SIGNAL_FUNC (edit_actions_entry_cb), NULL);
   gtk_widget_show (button);
   
   button = gtk_button_new_with_label (_("Remove"));
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (button), "clicked", 
 		      GTK_SIGNAL_FUNC (remove_actions_entry_cb), NULL);
   gtk_widget_show (button);
 
-  hbox = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_start (vbox, hbox, TRUE, TRUE, 0);
+  hbox = gtk_hbox_new (FALSE, 5);
+  gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
-  /* Add padding to right justify */
-  padding = gtk_label_new (" ");
-  gtk_widget_show (padding);
-  gtk_box_pack_start (GTK_BOX (hbox), padding, TRUE, TRUE, 0);
-
   button = gtk_button_new_from_stock (GNOME_STOCK_BUTTON_CANCEL);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
 			     GTK_OBJECT (actions_dialog));
   gtk_widget_show (button);
 
   button = gtk_button_new_from_stock (GNOME_STOCK_BUTTON_OK);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      GTK_SIGNAL_FUNC (apply_actions),
 		      NULL);
@@ -208,15 +190,12 @@ mon_edit_actions (GtkWidget *widget, gpointer data)
 void
 make_tree_from_actions_db (GList *db)
 {
-  char *text[2];
   char buffer[50];
   GList *item;
   GList *sibling = NULL;
   Action *action;
+  GtkTreeIter newiter;
 
-  text[0] = buffer;
-  text[1] = NULL;
-  
   /* Create first item */
   ctree_parent = NULL;
   buffer[0] = '\0';
@@ -228,16 +207,16 @@ make_tree_from_actions_db (GList *db)
 	  strncpy (buffer, action->tag, 50);
 	  buffer[49] = '\0';
 	  /* this is the parent node */
-	  sibling = (GList *)gtk_ctree_insert_node (ctree, NULL, NULL, (char **)text, 5, NULL, NULL,
-				      NULL, NULL, FALSE, FALSE);
-	  ctree_parent = sibling;
+          gtk_tree_store_append (ctree, &newiter, NULL);
+          gtk_tree_store_set (ctree, &newiter, 0, buffer, 1,
+                              (gpointer)action, -1);
+          ctree_parent = gtk_tree_iter_copy (&newiter);
 	}
       /* this is the actual node */
-      sibling = (GList *)gtk_ctree_insert_node (ctree, (GtkCTreeNode *)ctree_parent, NULL, (char **)text, 5, NULL, NULL,
-						NULL, NULL, FALSE, FALSE);
       /* Store data in tree */
-      gtk_ctree_node_set_row_data (ctree, (GtkCTreeNode *)sibling,
-				   (gpointer) action);
+      gtk_tree_store_append (ctree, &newiter, ctree_parent); 
+      gtk_tree_store_set (ctree, &newiter, 0, buffer, 1,
+                          (gpointer)action, -1);
     }
 
   /* done. */
@@ -523,16 +502,23 @@ static void
 remove_actions_entry_cb (GtkWidget *widget, gpointer data)
 {
   Action *action;
+  GtkTreeSelection *selection;
+  GtkTreeIter newiter;
+  GtkTreeModel *model = NULL;
+  gboolean selected;
 
-  if (selected_node == NULL)
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (ctree_view));
+  selected = gtk_tree_selection_get_selected (selection, &model, &newiter);
+  if (selected == FALSE)
 	  return;
 
-  action = (Action *) gtk_ctree_node_get_row_data (GTK_CTREE (ctree),
-						   selected_node);
+  selected_node = gtk_tree_iter_copy (&newiter);
+  gtk_tree_model_get (GTK_TREE_MODEL (model), selected_node, 1,
+		      (gpointer)&action, -1);
 
   if (action != NULL) {
 	  local_actions_db = g_list_remove (local_actions_db, action);
-	  gtk_clist_clear (GTK_CLIST (ctree));
+	  gtk_tree_store_clear (ctree);
 	  selected_node = NULL;
 	  make_tree_from_actions_db (local_actions_db);
   }
@@ -547,12 +533,19 @@ static void
 edit_actions_entry_cb (GtkWidget *widget, gpointer data)
 {
   Action *action;
+  GtkTreeSelection *selection;
+  GtkTreeIter newiter;
+  GtkTreeModel *model = NULL;
+  gboolean selected;
 
-  if (selected_node == NULL)
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (ctree_view));
+  selected = gtk_tree_selection_get_selected (selection, &model, &newiter);
+  if (selected == FALSE)
 	  return;
 
-  action = (Action *) gtk_ctree_node_get_row_data (GTK_CTREE (ctree),
-						   selected_node);
+  selected_node = gtk_tree_iter_copy (&newiter);
+  gtk_tree_model_get (GTK_TREE_MODEL (model), selected_node, 1,
+		      (gpointer)&action, -1);
 
   edit_action_entry (action);
 }
@@ -608,7 +601,7 @@ apply_edit (GtkWidget *w, gpointer data)
 	local_actions_db = g_list_sort (local_actions_db,
 					action_compare);
 
-	gtk_clist_clear (GTK_CLIST (ctree));
+	gtk_tree_store_clear (ctree);
 	selected_node = NULL;
 	make_tree_from_actions_db (local_actions_db);
 }
@@ -627,7 +620,6 @@ edit_action_entry (Action *action)
   GtkWidget *entry;
   GtkWidget *action_record;
   GtkWidget *button;       
-  GtkWidget *padding;       
   GtkBox *vbox;
   GtkTooltips *tips;
 
@@ -644,7 +636,7 @@ edit_action_entry (Action *action)
   gtk_container_set_border_width (GTK_CONTAINER (action_record), 5);
   gtk_window_set_title (GTK_WINDOW (action_record), "Edit action record");
   gtk_widget_set_style (action_record, cfg->main_style);
-  gtk_widget_set_usize (action_record, 400, 400);
+  gtk_widget_set_usize (action_record, 375, -1);
   
   vbox = (GtkBox *)gtk_vbox_new (FALSE, 2);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
@@ -656,7 +648,7 @@ edit_action_entry (Action *action)
 
   /* Tag */
   hbox = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_start (vbox, hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
   label = gtk_label_new ("Tag:");
   gtk_widget_set_usize (label, 60, -1);
@@ -673,7 +665,7 @@ edit_action_entry (Action *action)
 
   /* log name */
   hbox = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_start (vbox, hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
   label = gtk_label_new ("Log name:");
   gtk_widget_set_usize (label, 60, -1);
@@ -691,7 +683,7 @@ edit_action_entry (Action *action)
       
   /* Process */
   hbox = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_start (vbox, hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
   label = gtk_label_new ("process:");
   gtk_widget_set_usize (label, 60, -1);
@@ -709,7 +701,7 @@ edit_action_entry (Action *action)
 
   /* Message */
   hbox = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_start (vbox, hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
   label = gtk_label_new ("message:");
   gtk_widget_set_usize (label, 60, -1);
@@ -727,7 +719,7 @@ edit_action_entry (Action *action)
 
   /* Action */
   hbox = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_start (vbox, hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
   label = gtk_label_new ("action:");
   gtk_widget_set_usize (label, 60, -1);
@@ -745,15 +737,15 @@ edit_action_entry (Action *action)
 
   /* Description */
   hbox = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_start (vbox, hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
   label = gtk_label_new (_("description:"));
   gtk_widget_set_usize (label, 60, -1);
   gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
   gtk_widget_show (label);
-  text = gtk_text_view_new ();
+  text = gtk_entry_new ();
   gtk_widget_set_usize (text, 200, -1);
-  gtk_text_view_set_editable (GTK_TEXT_VIEW (text), TRUE);
+  gtk_entry_set_editable (GTK_ENTRY (text), TRUE);
   gtk_box_pack_start (GTK_BOX (hbox), text, TRUE, TRUE, 0);
   gtk_tooltips_set_tip (tips, text, _("Description of this entry."), NULL);
   gtk_widget_show (text); 
@@ -762,23 +754,18 @@ edit_action_entry (Action *action)
 
   /* Make bottom part ------------------------------------------------ */
   hbox = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_start (vbox, hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
-  /* Add padding to right justify */
-  padding = gtk_label_new (" ");
-  gtk_widget_show (padding);
-  gtk_box_pack_start (GTK_BOX (hbox), padding, TRUE, TRUE, 0);
-
   button = gtk_button_new_from_stock (GNOME_STOCK_BUTTON_CANCEL);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
 			     GTK_OBJECT (action_record));
   gtk_widget_show (button);
 
   button = gtk_button_new_from_stock (GNOME_STOCK_BUTTON_OK);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      GTK_SIGNAL_FUNC (apply_edit),
@@ -791,9 +778,10 @@ edit_action_entry (Action *action)
   gtk_widget_show (action_record);
 
   /* Insert text into text widget */
-  gtk_text_buffer_set_text (gtk_text_view_get_buffer(GTK_TEXT_VIEW (text)),
-  			    action->description, 
-	  		    strlen (action->description));
+  if (action->description)
+     gtk_text_buffer_set_text (gtk_text_view_get_buffer(GTK_TEXT_VIEW (text)),
+  			       action->description, 
+	  		       strlen (action->description));
 }
 
 /* ----------------------------------------------------------------------
